#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/image.h"
#include "wx/file.h"
#include "wx/statline.h"
#include "measurement.cpp"

wxFile *logFile;
// fileok betolteset jelzo flagek
bool imageLoaded = false, vpLoaded = false, addingVPs = false;
// kep meretei
int width,height;
// vanishing pointok szama
int vpCount = 0;
// vanishing pointok
long **vanishingPoints = NULL;
// kivalasztott 2 vp
long selectedVP[2][2] = { {-1,-1} , {-1,-1} };
// referencia objektum merete
float refSize = -1.0;
// referencia objektum
long refObject[2][2] = { {-1,-1} , {-1,-1} };
// egerrel kivalasztott aktualis vonal
int mouseLine[2][2] = { {-1,-1} , {-1,-1} };

// ----------------------------------------------------------------------------
// SvmApp - az alkalmazashoz tartozo osztaly
// ----------------------------------------------------------------------------

class SvmApp: public wxApp {
	// ez hivodik meg az alkalmazas indulasakor
	virtual bool OnInit();
};


// ----------------------------------------------------------------------------
// BitmapCanvas - a MainPanel tartalma, a megjelenitest vegzi
// ----------------------------------------------------------------------------

class BitmapCanvas : public wxScrolledWindow {
public:
	// konstruktorok
	BitmapCanvas(wxWindow *parent, const wxSize& size);
	// a bitmapet rendeli hozza az ablakhoz
	void SetBitmap(const wxBitmap& bitmap);
	// bitmapet adja vissza
	wxBitmap* GetBitmap();
	// a kep megjelenitese
	void OnPaint(wxPaintEvent& event);
	// eger kezeles
	void OnMouseUp(wxMouseEvent& event);
	void OnMouseDown(wxMouseEvent& event);
	void OnMouseMove(wxMouseEvent& event);
protected:
	// esemenytabla (az OnPainthez)
	DECLARE_EVENT_TABLE()
private:
	// eger lenyomasat jelzo flag
	bool mouseDown;
	// kezdeti uzenet
	wxStaticText *labelWarning;
	// a bitmap
	wxBitmap m_bitmap;
};

// ----------------------------------------------------------------------------
// RadioWindow - a MainPanel tartalma, a radio gombokat tartalmazza
// ----------------------------------------------------------------------------

class RadioWindow : public wxScrolledWindow {
public:
	// konstruktorok
	RadioWindow(wxWindow *parent, const wxSize& size);
	// radio gombok epitese
	void CreateContent(const char* vpFileName, int x, int y);
	// radio gomb megnyomasakor hivodik meg
	void OnRadio1(wxCommandEvent& event);
	void OnRadio2(wxCommandEvent& event);
protected:
	DECLARE_EVENT_TABLE()
private:
	// kezdeti szoveg
	wxStaticText *labelWarning;
	// radio boxok
	wxRadioBox *radioBox1;
	wxRadioBox *radioBox2;

	enum {
		RADIO_1 = 1,
		RADIO_2,
	};
};

// ----------------------------------------------------------------------------
// ControlWindow - a MainPanel tartalma, fo kezelo szerveket tartalmazza
// ----------------------------------------------------------------------------

class ControlWindow : public wxPanel {
public:
	// konstruktorok
	ControlWindow(wxWindow *parent, const wxSize& size);
	// esemenykezelok
	void OnTextRefsize(wxCommandEvent& event);
	void OnBtnStartAdd(wxCommandEvent& event);
	void OnBtnStopAdd(wxCommandEvent& event);
	void OnBtnDefref(wxCommandEvent& event);
	void OnBtnMeasure(wxCommandEvent& event);
	// gombokat aktivalja ha lehetseges, egyebkent deaktivalja
	void EnableBtns();
	// get fuggvenyek
	wxButton* GetBtnMeasure();
	wxButton* GetBtnDefref();
protected:
	DECLARE_EVENT_TABLE()
private:
	wxTextCtrl *textRefsize, *textResult;
	wxButton *btnDefref, *btnMeasure, *btnStartAdd, *btnStopAdd;

	enum {
		TEXT_REFSIZE = 11,
		TEXT_RESULT,
		BTN_STARTADD,
		BTN_STOPADD,
		BTN_DEFREF,
		BTN_MEASURE,
	};
};

// ----------------------------------------------------------------------------
// MainPanel - foablak tartalma
// ----------------------------------------------------------------------------

class MainPanel: public wxPanel {
public:
	// konstruktor
	MainPanel(wxFrame *parent);
	// get fuggvenyek
	BitmapCanvas* GetCanvas();
	RadioWindow* GetRadioWindow();
	ControlWindow* GetControlWindow();
private:
	BitmapCanvas *canvas;
	RadioWindow *radioWindow;
	ControlWindow *controlWindow;
};

// ----------------------------------------------------------------------------
// MainFrame - foablak
// ----------------------------------------------------------------------------

class MainFrame: public wxFrame {
public:
	// konstruktor
	MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size);
	// a Kep Megnyitasa menure kattintva hivodik meg
	void OnOpenImage(wxCommandEvent& event);
	// a VP File Megnyitasa menure kattintva hivodik meg
	void OnOpenVp(wxCommandEvent& event);
	// a Kilepes menure kattintva hivodik meg
	void OnQuit(wxCommandEvent& event);
	// a Nevjegy menure kattintva hivodik meg
	void OnAbout(wxCommandEvent& event);
	// menu epitese
	void MenuBuild(int open);
protected:
	DECLARE_EVENT_TABLE()
private:
	wxMenuBar *menuBar;
	wxMenu *fileMenu, *infoMenu;
	MainPanel *mainPanel;
	wxImage image;

	enum {
		MENU_IMAGE_OPEN = 21,
		MENU_VPFILE_OPEN,
		MENU_FILE_QUIT,
		MENU_INFO_ABOUT,
	};
};

// ----------------------------------------------------------------------------
// VPFIleReader - Vanishing Point fileok beolvasasara
// ----------------------------------------------------------------------------

class VPFileReader {
public:
	// konstruktor, destruktor
	VPFileReader(const char *vpFileName);
	~VPFileReader();
	// beolvasas
	void ReadVPFile();
private:
	char *vpFileName;
	wxFile *vpFile;
};


// az esemenytablak (esemenyekhez fuggvenyeket rendel hozza)

BEGIN_EVENT_TABLE(BitmapCanvas, wxScrolledWindow)
	EVT_PAINT(BitmapCanvas::OnPaint)
	EVT_LEFT_DOWN(BitmapCanvas::OnMouseDown)
	EVT_LEFT_UP(BitmapCanvas::OnMouseUp)
	EVT_MOTION(BitmapCanvas::OnMouseMove)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(RadioWindow, wxScrolledWindow)
	EVT_RADIOBOX(RADIO_1, RadioWindow::OnRadio1)
	EVT_RADIOBOX(RADIO_2, RadioWindow::OnRadio2)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(ControlWindow, wxPanel)
	EVT_TEXT(TEXT_REFSIZE, ControlWindow::OnTextRefsize)
	EVT_BUTTON(BTN_STARTADD, ControlWindow::OnBtnStartAdd)
	EVT_BUTTON(BTN_STOPADD, ControlWindow::OnBtnStopAdd)
	EVT_BUTTON(BTN_DEFREF, ControlWindow::OnBtnDefref)
	EVT_BUTTON(BTN_MEASURE, ControlWindow::OnBtnMeasure)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
	EVT_MENU(MENU_IMAGE_OPEN, MainFrame::OnOpenImage)
	EVT_MENU(MENU_VPFILE_OPEN, MainFrame::OnOpenVp)
	EVT_MENU(MENU_FILE_QUIT, MainFrame::OnQuit)
	EVT_MENU(MENU_INFO_ABOUT, MainFrame::OnAbout)
END_EVENT_TABLE()


// ============================================================================
// ============================================================================




// ----------------------------------------------------------------------------
// SvmApp
// ----------------------------------------------------------------------------

IMPLEMENT_APP(SvmApp)

bool SvmApp::OnInit() {
	// foablak letrehozasa
	MainFrame *frame = new MainFrame( "Svm", wxPoint(0,0), wxSize(1024,640) );
	// es megmutatasa
	frame->Show( true );
	SetTopWindow( frame );
	return true;
}



// ----------------------------------------------------------------------------
// BitmapCanvas
// ----------------------------------------------------------------------------

BitmapCanvas::BitmapCanvas(wxWindow *parent, const wxSize& size) : wxScrolledWindow(parent, -1, wxDefaultPosition, size) {
	mouseDown = false;

	// az ablak letrehozasa
	wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

	labelWarning = new wxStaticText(this,-1,"No image file opened!");
	mainSizer->Add(labelWarning, 0, wxTOP | wxALIGN_CENTER, 50);

	// a sizer beallitasa
	SetSizer(mainSizer);
}

void BitmapCanvas::SetBitmap(const wxBitmap& bitmap) {
	// ablak torlese
	if (!imageLoaded) labelWarning->Destroy();
	// a bitmap betoltese
	m_bitmap = bitmap;
	imageLoaded = true;
	// a scrollbar meretenek es leptekenek beallitasa
	SetScrollbars(10, 10,m_bitmap.GetWidth() / 10, m_bitmap.GetHeight() / 10);
	// frissites
	Refresh(true);
	Update();
}

wxBitmap* BitmapCanvas::GetBitmap() {
	return &m_bitmap;
}

void BitmapCanvas::OnPaint(wxPaintEvent& event) {
	// device context letrehozasa es hozzarendelese az ablakhoz
	wxPaintDC dc(this);
	PrepareDC(dc);
	// a kep megjelenitese
	if (m_bitmap.Ok()) {
		dc.DrawBitmap(m_bitmap, 0, 0);
	}

	// ha van megadva ref. obj. akkor azt is kirajzolja
	if (!((refObject[0][0] == -1) || (refObject[1][0] == -1) || (refObject[0][1] == -1) || (refObject[1][1] == -1) || (refSize == -1.0))) {
		dc.SetPen(*wxRED_PEN);
		dc.DrawLine(refObject[0][0],refObject[0][1],refObject[1][0],refObject[1][1]);
		dc.SetPen(wxPen(wxColour(255,100,100),1,wxSOLID));
		dc.DrawLine(refObject[0][0]-3,refObject[0][1]-3,refObject[0][0]+3,refObject[0][1]+3);
		dc.DrawLine(refObject[0][0]+3,refObject[0][1]-3,refObject[0][0]-3,refObject[0][1]+3);
		dc.DrawLine(refObject[1][0]-3,refObject[1][1]-3,refObject[1][0]+3,refObject[1][1]+3);
		dc.DrawLine(refObject[1][0]+3,refObject[1][1]-3,refObject[1][0]-3,refObject[1][1]+3);
	}

	// vanishing pointok kirajzolasa
	if ((selectedVP[0][0] >= 0) && (selectedVP[0][0] < width) && (selectedVP[0][1] >= 0) && (selectedVP[0][1] < height) && (imageLoaded)) {
		dc.SetPen(wxPen(wxColour(0,255,100),3,wxSOLID));
		dc.DrawLine(selectedVP[0][0]-3,selectedVP[0][1]-3,selectedVP[0][0]+3,selectedVP[0][1]+3);
		dc.DrawLine(selectedVP[0][0]+3,selectedVP[0][1]-3,selectedVP[0][0]-3,selectedVP[0][1]+3);
	}
	if ((selectedVP[1][0] >= 0) && (selectedVP[1][0] < width) && (selectedVP[1][1] >= 0) && (selectedVP[1][1] < height) && (imageLoaded)) {
		dc.SetPen(wxPen(wxColour(0,255,100),3,wxSOLID));
		dc.DrawLine(selectedVP[1][0]-3,selectedVP[1][1]-3,selectedVP[1][0]+3,selectedVP[1][1]+3);
		dc.DrawLine(selectedVP[1][0]+3,selectedVP[1][1]-3,selectedVP[1][0]-3,selectedVP[1][1]+3);
	}
	// vanishing line kirajzolasa
	if ((selectedVP[0][0] != -1) && (selectedVP[0][1] != -1) && (selectedVP[1][0] != -1) && (selectedVP[1][1] != -1)) {
		int x1 = selectedVP[0][0];
		int y1 = selectedVP[0][1];
		int x2 = selectedVP[1][0];
		int y2 = selectedVP[1][1];

		// megcsereli a ket pontot ha az elso x koordinataja nagyobb
		if (x1 > x2) { int tmp=x1; x1=x2; x2=tmp; tmp=y1; y1=y2; y2=tmp;}

		int X1=x1, Y1=y1, X2=x2, Y2=y2;
		
		// addig huzza szet a vanishing point szakaszat amig nem megy vegig a kepen
		if (!((x1==x2) && (y1==y2))) {
			while (x1>0)		{ x1-=(X2-X1); y1-=(Y2-Y1); }
			while (x2<width)	{ x2+=(X2-X1); y2+=(Y2-Y1); }
		}

//		logFile->Write(wxString::Format(_T("x1:%d y1:%d,  x2:%d y2:%d\n"),x1,y1,x2,y2));
//		logFile->Write(wxString::Format(_T("X1:%d Y1:%d,  X2:%d Y2:%d\n"),X1,Y1,X2,Y2));
//		logFile->Write(wxString::Format(_T("X2-X1:%d Y2-Y1:%d\n"),X2-X1,Y2-Y1));
		dc.SetPen(wxPen(wxColour(0,192,100),2,wxSOLID));
		dc.DrawLine(x1,y1,x2,y2);
	}
	
	// eger altal kijelolt vonal kirajzolasa
	if (!((mouseLine[0][0] == -1) || (mouseLine[1][0] == -1) || (mouseLine[0][1] == -1) || (mouseLine[1][1] == -1))) {
		dc.SetPen(wxPen(wxColour(0,128,210),1,wxSOLID));
		dc.DrawLine(mouseLine[0][0], mouseLine[0][1], mouseLine[1][0], mouseLine[1][1]);
	}
}

void BitmapCanvas::OnMouseDown(wxMouseEvent& event) {
	if (!addingVPs) {
		mouseDown = true;
		
		int x,y;
		CalcUnscrolledPosition(event.GetX(),event.GetY(),&x,&y);
		mouseLine[0][0] = x;
		mouseLine[0][1] = y;
//	logFile->Write(wxString::Format(_T("+ Down: %d %d\n"),mouseLine[0][0],mouseLine[0][1]));
	}
}

void BitmapCanvas::OnMouseUp(wxMouseEvent& event) {
	int x,y;
	CalcUnscrolledPosition(event.GetX(),event.GetY(),&x,&y);

	if (!addingVPs) {
		mouseDown = false;
		
		mouseLine[1][0] = x;
		mouseLine[1][1] = y;
	//	logFile->Write(wxString::Format(_T("- Up: %d %d\n"),mouseLine[1][0],mouseLine[1][1]));
		// ablak tartalmanak frissitese
		Refresh();
		// defref gomb aktivalasa ha lehetseges
		((MainPanel*)GetParent())->GetControlWindow()->EnableBtns();
	} else {
		((MainPanel*)GetParent())->GetRadioWindow()->CreateContent(NULL,x,y);
	}
}

void BitmapCanvas::OnMouseMove(wxMouseEvent& event) {
	int x,y;
	CalcUnscrolledPosition(event.GetX(),event.GetY(),&x,&y);
	((MainFrame*)(GetParent()->GetParent()))->SetStatusText(wxString::Format(_T("%d,%d"),x,y), 4);

	if (mouseDown) {
		mouseLine[1][0] = x;
		mouseLine[1][1] = y;
		// ablak tartalmanak frissitese
		Refresh();
//		logFile->Write(wxString::Format(_T("\tMove: %d %d\n"),mouseLine[1][0],mouseLine[1][1]));
	}
}

// ----------------------------------------------------------------------------
// RadioWindow
// ----------------------------------------------------------------------------

RadioWindow::RadioWindow(wxWindow *parent, const wxSize& size) : wxScrolledWindow(parent, -1, wxDefaultPosition, size, wxSTATIC_BORDER) {
	// az ablak letrehozasa
	wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

	labelWarning = new wxStaticText(this,-1,"No Vanishing Points file opened!");
	mainSizer->Add(labelWarning, 0, wxTOP | wxALIGN_CENTER, 50);

	// a sizer beallitasa
	SetSizer(mainSizer);
}

void RadioWindow::CreateContent(const char* vpFileName, int x, int y) {
	// ablak torlese
	if (!vpLoaded) labelWarning->Destroy();
	else {
		radioBox1->Destroy();
		radioBox2->Destroy();
	}

	if ((x==0) && (y==0)) { // ha file-t toltunk be
		// VP file beolvasasa
		VPFileReader *vpReader = new VPFileReader(vpFileName);
		vpReader->ReadVPFile();
		delete vpReader;
	} else { // ha kezzel adunk meg vp-t
		vpCount++;

		// uj lista keszitese
		long **temp = new long*[vpCount];
		for (int i=0;i<vpCount;i++) {
			temp[i] = new long[2];
			if (i<vpCount-1) {
				temp[i][0] = vanishingPoints[i][0];
				temp[i][1] = vanishingPoints[i][1];
			} else {
				temp[i][0] = x;
				temp[i][1] = y;
			}
		}

		for (int j=0;j<vpCount-1;j++) delete vanishingPoints[j];
		delete vanishingPoints;

		vanishingPoints = temp;
		if (vpCount==1) {
			selectedVP[0][0] = selectedVP[1][0] = vanishingPoints[0][0];
			selectedVP[0][1] = selectedVP[1][1] = vanishingPoints[0][1];
		}

		vpLoaded = true;

		((MainFrame*)(GetParent()->GetParent()))->SetStatusText(wxString::Format(_T("Vanishing Point (%d,%d) added"),x,y), 0);
	}

	// nevlista feltoltese
	wxString *vpArray = new wxString[vpCount];
	for (int i=0;i<vpCount;i++) vpArray[i] = wxString::Format(_T("%ld,%ld"),vanishingPoints[i][0],vanishingPoints[i][1]);

	// measure gomb aktivalasa ha lehetseges
	((MainPanel*)GetParent())->GetControlWindow()->EnableBtns();

	// az ablak letrehozasa
	wxBoxSizer *mainSizer = new wxBoxSizer(wxHORIZONTAL);

	radioBox1 = new wxRadioBox(this, RADIO_1, "VP 1:", wxDefaultPosition, wxDefaultSize, vpCount, vpArray, 1, wxRA_SPECIFY_COLS);
	radioBox2 = new wxRadioBox(this, RADIO_2, "VP 2:", wxDefaultPosition, wxDefaultSize, vpCount, vpArray, 1, wxRA_SPECIFY_COLS);
	mainSizer->Add(radioBox1, 1, wxEXPAND | wxALL, 5);
	mainSizer->Add(radioBox2, 1, wxEXPAND | wxALL, 5);

	// a sizer beallitasa
	SetSizer(mainSizer);

	int *w = new int;
	int *h = new int;
	GetSize(w,h);
//	logFile->Write(wxString::Format(_T("%d %d"),(*w)/10,(*h)/10));
	SetScrollbars(10, 10,(*w) / 10, (*h) / 10);

	delete w; delete h;
}

void RadioWindow::OnRadio1(wxCommandEvent& event) {
	int radioSel = event.GetSelection();

	selectedVP[0][0] = vanishingPoints[radioSel][0];
	selectedVP[0][1] = vanishingPoints[radioSel][1];
//	logFile->Write(wxString::Format(_T("1: (%d) %ld %ld\n"),radioSel,selectedVP[0][0],selectedVP[0][1]));

	// measure gomb aktivalasa ha lehetseges
	((MainPanel*)GetParent())->GetControlWindow()->EnableBtns();

	// rajzolo ablak frissitese ha van betoltve kep
	if (imageLoaded) ((MainPanel*)GetParent())->GetCanvas()->Refresh();
}

void RadioWindow::OnRadio2(wxCommandEvent& event) {
	int radioSel = event.GetSelection();

	selectedVP[1][0] = vanishingPoints[radioSel][0];
	selectedVP[1][1] = vanishingPoints[radioSel][1];
//	logFile->Write(wxString::Format(_T("2: (%d) %ld %ld\n"),radioSel,selectedVP[1][0],selectedVP[1][1]));

	// measure gomb aktivalasa ha lehetseges
	((MainPanel*)GetParent())->GetControlWindow()->EnableBtns();

	// rajzolo ablak frissitese ha van betoltve kep
	if (imageLoaded) ((MainPanel*)GetParent())->GetCanvas()->Refresh();
}

// ----------------------------------------------------------------------------
// ControlWindow
// ----------------------------------------------------------------------------

ControlWindow::ControlWindow(wxWindow *parent, const wxSize& size) : wxPanel(parent, -1, wxDefaultPosition, size, wxSTATIC_BORDER) {
	// az ablak letrehozasa
	wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

	// vanishing point hozzaadas gombok
	wxBoxSizer *vpSizer = new wxBoxSizer(wxHORIZONTAL);
	btnStartAdd = new wxButton(this, BTN_STARTADD, "Add Vanishing Points");
	vpSizer->Add(btnStartAdd, 0, wxRIGHT, 3);
	btnStopAdd = new wxButton(this, BTN_STOPADD, "Stop Adding");
	vpSizer->Add(btnStopAdd, 1);

	mainSizer->Add(vpSizer, 0, wxALIGN_CENTER | wxTOP, 5);

	// elvalaszto vonal
	wxStaticLine *line0 = new wxStaticLine(this, -1, wxDefaultPosition, wxSize(200,-1), wxLI_HORIZONTAL);
	mainSizer->Add(line0, 0, wxTOP, 5);

	// referenciameret beviteli mezo
	wxBoxSizer *inputSizer = new wxBoxSizer(wxHORIZONTAL);
	wxStaticText *labelRefsize = new wxStaticText(this,-1,"Ref. object size: ");
	inputSizer->Add(labelRefsize, 0, wxTOP, 5);
	textRefsize = new wxTextCtrl(this, TEXT_REFSIZE);
	inputSizer->Add(textRefsize, 1);

	mainSizer->Add(inputSizer, 0, wxALIGN_CENTER | wxTOP, 5);

	// define ref object gomb
	btnDefref = new wxButton(this, BTN_DEFREF, "Define ref. object");
	mainSizer->Add(btnDefref, 0, wxALIGN_CENTER | wxTOP, 5);

	// elvalaszto vonal
	wxStaticLine *line1 = new wxStaticLine(this, -1, wxDefaultPosition, wxSize(200,-1), wxLI_HORIZONTAL);
	mainSizer->Add(line1, 0, wxTOP, 5);

	// measure gomb
	btnMeasure = new wxButton(this, BTN_MEASURE, "Measure");
	mainSizer->Add(btnMeasure, 0, wxALIGN_CENTER | wxALL, 5 );
	
	// elvalaszto vonal
	wxStaticLine *line2 = new wxStaticLine(this, -1, wxDefaultPosition, wxSize(200,-1), wxLI_HORIZONTAL);
	mainSizer->Add(line2, 0);

	// eredmeny kijelzo mezo
	wxBoxSizer *resultSizer = new wxBoxSizer(wxHORIZONTAL);
	wxStaticText *labelResult = new wxStaticText(this,-1,"Measurement: ");
	resultSizer->Add(labelResult, 0, wxTOP, 5);
	textResult = new wxTextCtrl(this, TEXT_RESULT, "", wxDefaultPosition, wxSize(110,-1), wxTE_RICH | wxTE_READONLY | wxTE_CENTRE);
	textResult->SetDefaultStyle(wxTextAttr(wxColour(192,0,0),wxColour(128,128,128),wxFont(10,wxDECORATIVE,wxNORMAL,wxBOLD)));
	textResult->WriteText("nothing");
	textResult->Disable();
	resultSizer->Add(textResult, 1);

	mainSizer->Add(resultSizer, 0, wxALIGN_CENTER | wxTOP, 10);

	// a sizer beallitasa
	SetSizer(mainSizer);

	EnableBtns();
}

void ControlWindow::OnTextRefsize(wxCommandEvent& event) {
	double val;
	
	if (textRefsize->GetValue().ToDouble(&val)) {
		refSize = (float)val;
	} else {
		refSize = -1.0;
	}

	// measure gomb aktivalasa ha lehetseges
	EnableBtns();
}

void ControlWindow::OnBtnStartAdd(wxCommandEvent& event) {
	addingVPs = true;
	EnableBtns();
}

void ControlWindow::OnBtnStopAdd(wxCommandEvent& event) {
	addingVPs = false;
	EnableBtns();
}

void ControlWindow::OnBtnDefref(wxCommandEvent& event) {
	// referencia objektum megadasa
	refObject[0][0] = mouseLine[0][0];
	refObject[0][1] = mouseLine[0][1];
	refObject[1][0] = mouseLine[1][0];
	refObject[1][1] = mouseLine[1][1];

	// statuszsor beallitasa
	char *str = new char[256];
	sprintf(str,"RefObj: (%d,%d)-(%d,%d) (%.2f)",refObject[0][0],refObject[0][1],refObject[1][0],refObject[1][1],refSize);
	((MainFrame*)(GetParent()->GetParent()))->SetStatusText("Reference object defined succesfully", 0);
	((MainFrame*)(GetParent()->GetParent()))->SetStatusText(str, 3);
	delete str;

	// rajzolo ablak frissitese
	((MainPanel*)GetParent())->GetCanvas()->Refresh();
	// measure gomb aktivalasa ha lehetseges
	EnableBtns();
}

void ControlWindow::OnBtnMeasure(wxCommandEvent& event) {
	Measurement *measure = new Measurement(width,height,vanishingPoints[0][0],vanishingPoints[0][1],vanishingPoints[1][0],vanishingPoints[1][1]);

	measure->setRefData(refObject[0][0],refObject[0][1],refObject[1][0],refObject[1][1],refSize);
	measure->setMeasureCoordinates(mouseLine[0][0],mouseLine[0][1],mouseLine[1][0],mouseLine[1][1]);
	textResult->SetValue(wxString::Format(_T("%.2f"),measure->calculateSize()));

	delete measure;
}

void ControlWindow::EnableBtns() {
	bool measure = true, defref = true, startadd = true, stopadd = true;

	// ref. obj. meretenek beallitasa 1-re ha meg nincs beallitva (nem kell mindig beirni ha csak aranyokat akarunk)
	if (refSize == -1.0) textRefsize->SetValue("1.0");

	// kizaro okok
	if ((selectedVP[0][0] == selectedVP[1][0]) && (selectedVP[0][1] == selectedVP[1][1])) measure=false;
	if ((refObject[0][0] == -1) || (refObject[1][0] == -1) || (refObject[0][1] == -1) || (refObject[1][1] == -1)) measure=false;
	if (!vpLoaded) measure=false;
	if (addingVPs) measure=false;

	if (!imageLoaded) defref = false;
	if (refSize == -1.0) defref = false;
	if (addingVPs) defref=false;

	if (addingVPs) startadd=false;
	if (!imageLoaded) startadd=false;
	if (!addingVPs) stopadd=false;

	btnMeasure->Enable(measure);
	btnDefref->Enable(defref);
	btnStartAdd->Enable(startadd);
	btnStopAdd->Enable(stopadd);
}

wxButton* ControlWindow::GetBtnMeasure() {
	return btnMeasure;
}

wxButton* ControlWindow::GetBtnDefref() {
	return btnDefref;
}

// ----------------------------------------------------------------------------
// MainPanel
// ----------------------------------------------------------------------------

MainPanel::MainPanel(wxFrame *parent) : wxPanel(parent,-1) {
	// az ablak letrehozasa
	wxBoxSizer *mainSizer = new wxBoxSizer(wxHORIZONTAL);

	canvas = new BitmapCanvas(this,wxDefaultSize);
	mainSizer->Add(canvas, 1, wxEXPAND);

	wxBoxSizer *rightSizer = new wxBoxSizer(wxVERTICAL);

	radioWindow = new RadioWindow(this, wxSize(200,-1));
	rightSizer->Add(radioWindow, 1, wxEXPAND | wxALL, 5);

	controlWindow = new ControlWindow(this, wxSize(190,170));
	rightSizer->Add(controlWindow, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 5);

	mainSizer->Add(rightSizer, 0, wxEXPAND);

	// a sizer beallitasa
	SetSizer(mainSizer);
}

BitmapCanvas* MainPanel::GetCanvas() {
	return canvas;
}

RadioWindow* MainPanel::GetRadioWindow() {
	return radioWindow;
}

ControlWindow* MainPanel::GetControlWindow() {
	return controlWindow;
}

// ----------------------------------------------------------------------------
// MainFrame
// ----------------------------------------------------------------------------

MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size)	: wxFrame((wxFrame *)NULL, -1, title, pos, size,wxMINIMIZE_BOX | wxMAXIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION) {
	// inicializalas
	menuBar = NULL;
	fileMenu = infoMenu = NULL;

	// kezelo raallitasa a logfilera
//	logFile = new wxFile("log.txt",wxFile::write);

	// statuszsor letrehozasa
	CreateStatusBar(5);
	int ratios[] = {-4,-3,-3,-3,-1};
	SetStatusWidths(5,ratios);
	// menu letrehozasa
	MenuBuild(1);

	// ablak tartalmanak letrehozasa
	mainPanel = new MainPanel(this);
}

void MainFrame::OnOpenImage(wxCommandEvent& event) {
	// statuszsort beallito string
	char *str = new char[256];
	// az ablak letrehozasa
	wxFileDialog *dlg = new wxFileDialog(this, "Open image","","", "Image files|*.bmp;*.jpg;*.jpeg;*.gif;*.png;*.tiff;*.tif;*.xpm;*.pcx|All files|*.*",wxOPEN, wxDefaultPosition);

	// ha a megnyitas ablak megjelent
	if ( dlg->ShowModal() == wxID_OK ) {
		// handlerek betoltese
		wxInitAllImageHandlers();
		// kep betoltese
		image.LoadFile(dlg->GetPath());

		mainPanel->GetCanvas()->SetBitmap(wxBitmap(image));

		// meretet tartalmazo valtozok beallitasa
		width = image.GetWidth();
		height = image.GetHeight();
		// statussor beallitasa
		sprintf(str,"Image: %s (%dx%d)",dlg->GetFilename(),image.GetWidth(),image.GetHeight());
		SetStatusText("Image file opened succesfully", 0);
		SetStatusText(str, 1);
	}
	delete str;
	dlg->Destroy();
}

void MainFrame::OnOpenVp(wxCommandEvent& event) {
	// statussort beallito string
	char *str = new char[256];

	// tallozo ablak letrehozasa
	wxFileDialog *dlg = new wxFileDialog(this, "Open Vanishing Points file","","", "Vanishing Points files|*.dat;*.vp|All files|*.*",wxOPEN, wxDefaultPosition);

	// ha a megnyitas ablak megjelent
	if ( dlg->ShowModal() == wxID_OK ) {
		mainPanel->GetRadioWindow()->CreateContent((dlg->GetPath()).GetData(),0,0);

		// statussor beallitasa
		sprintf(str,"VPFile: %s",dlg->GetFilename());
		SetStatusText("Vanishing Points file opened succesfully", 0);
		SetStatusText(str, 2);
	}
	delete str;
	dlg->Destroy();
}

void MainFrame::OnQuit(wxCommandEvent& event) {
	// logfile bezarasa
//	logFile->Close();
//	delete logFile;
	// foablak bezarasa
	Close(true);
}

void MainFrame::OnAbout(wxCommandEvent& event) {
	// nevjegy eloallitasa
	wxMessageBox( "Demonstration program for\n  Single View Metrology method.\n\n\tJuhasz Reka\n\tPapp Laszlo\n\tPinter Csaba\n\tSoponyai Gyorgy\n\nDate: 2005.11.26","Svm info", wxOK | wxICON_INFORMATION );
}

void MainFrame::MenuBuild(int open) {
	menuBar = new wxMenuBar();
	fileMenu = new wxMenu();
	fileMenu->Append(MENU_IMAGE_OPEN, "&Open Image...\tCtrl-O", "Open image file");
	fileMenu->Append(MENU_VPFILE_OPEN, "&Open VP File...\tCtrl-P", "Open file containing Vanishing Points");
	fileMenu->AppendSeparator();
	fileMenu->Append(MENU_FILE_QUIT, "&Quit\tAlt-F4", "Quit program");
	menuBar->Append(fileMenu, "&File");

	infoMenu = new wxMenu();
	infoMenu->Append(MENU_INFO_ABOUT, "&About", "About the program");
	menuBar->Append(infoMenu, "&Help");
	SetMenuBar(menuBar);
}

// ----------------------------------------------------------------------------
// VPFIleReader
// ----------------------------------------------------------------------------

VPFileReader::VPFileReader(const char* fileName) {
	// inicializalas
	vpFileName = (char *)fileName;

	// kezelo raallitasa a logfilera
	vpFile = new wxFile(fileName,wxFile::read);
}

VPFileReader::~VPFileReader() {
	vpFile->Close();
	delete vpFile;
}

void VPFileReader::ReadVPFile() {
	wxString str, strCp, temp;
	long tmpVp1, tmpVp2;

	// ha vannak betoltve vp-k, akkor azok torlese
	if (vanishingPoints) {
		for (int i=0;i<vpCount;i++) delete vanishingPoints[i];
		delete vanishingPoints;
	}

	// file beolvasasa stringbe
	vpFile->Read(str.GetWriteBuf(vpFile->Length()),vpFile->Length());
	strCp = str.Left(str.Length());

	// vanishing pointok szamanak meghatarozasa
	vpCount = 0;
	do {
		temp = str.BeforeFirst('\x0a');
		str = str.AfterFirst('\x0a');
		vpCount++;
	} while (temp.Length() > 0);
	vpCount--;

	// vanishing pointok kinyerese
	str = strCp.Left(strCp.Length());

	vanishingPoints = new long*[vpCount];
	for (int i=0;i<vpCount;i++) vanishingPoints[i] = new long[2];

	for (int j=0;j<vpCount;j++) {
		temp = str.BeforeFirst('\x0a');
		temp = temp.Left(temp.Length()-1);
		str = str.AfterFirst('\x0a');

		temp.BeforeFirst('\x20').ToLong(&tmpVp1,10);
		vanishingPoints[j][0] = tmpVp1;
		temp.AfterFirst('\x20').ToLong(&tmpVp2,10);
		vanishingPoints[j][1] = tmpVp2;
	}

	vpLoaded = true;
	selectedVP[0][0] = selectedVP[1][0] = vanishingPoints[0][0];
	selectedVP[0][1] = selectedVP[1][1] = vanishingPoints[0][1];
}